@@ -14,18 +14,25 @@
  * (enclosed in the file GPL).
  *)
 
-(*i $Id: biboutput.ml,v 1.10 2001/07/13 12:37:57 filliatr Exp $ i*)
+(*i $Id: biboutput.ml,v 1.14 2004/03/24 08:10:10 filliatr Exp $ i*)
 
 (*s Output a BibTeX bibliography. *)
 
 open Printf
 open Bibtex
 
+let link_fields = ref []
+let add_link_field f = link_fields := f :: !link_fields
+let is_link_field f = List.mem f !link_fields
+
 let needs_output k = function
   | None -> true
   | Some s -> KeySet.mem k s
 
-let print_atom html ch keys = function
+let url_re = Str.regexp "\\(ftp\\|http\\)://.*"
+let is_url s = Str.string_match url_re s 0
+
+let print_atom html ch = function
   | Id s -> 
       if html & not (abbrev_is_implicit s) then 
 	begin
@@ -35,18 +42,24 @@ let print_atom html ch keys = function
 	end
       else
 	output_string ch s
+  | String s when html & is_url s -> 
+      output_string ch "{";
+      Html.open_href ch s;
+      output_string ch s;
+      Html.close_href ch;
+      output_string ch "}"
   | String s -> 
       output_string ch ("{"^s^"}")
 
-let print_atom_list html ch keys = function 
+let print_atom_list html ch = function 
   | [] -> ()
   | a::l ->
-      print_atom html ch keys a;
+      print_atom html ch a;
       List.iter
-	(fun a -> output_string ch " # "; print_atom html ch keys a)
+	(fun a -> output_string ch " # "; print_atom html ch a)
 	l
 
-let print_crossref html ch keys = function
+let print_crossref html ch = function
   | [String s] -> 
       output_string ch "{";
       if html then Html.open_href ch ("#" ^ s);
@@ -56,7 +69,19 @@ let print_crossref html ch keys = function
   | l -> 
       if not !Options.quiet then
 	eprintf "Warning: cross-references must be constant strings\n";
-      print_atom_list html ch keys l
+      print_atom_list html ch l
+
+let print_link_field ch = function
+  | [String s] ->
+      output_string ch "{";
+      Html.open_href ch s;
+      output_string ch s;
+      Html.close_href ch;
+      output_string ch "}"
+  | l ->
+      if not !Options.quiet then
+	eprintf "Warning: web links must be constant strings\n";
+      print_atom_list true ch l
 
 let print_command html ch keys = function 
   | Comment s -> 
@@ -69,29 +94,48 @@ let print_command html ch keys = function
 	end
       else
 i*)
-	output_string ch ("@COMMENT{{" ^ s ^ "}}\n\n")
-  | Preamble s ->
-      output_string ch ("@PREAMBLE{{" ^ s ^ "}}\n\n")
+      if html then output_string ch "<pre>\n";
+      output_string ch ("@COMMENT{{" ^ s ^ "}}\n");
+      if html then output_string ch "</pre>\n";
+      output_string ch "\n"
+  | Preamble l ->
+      if html then output_string ch "<pre>\n";
+      output_string ch "@PREAMBLE{{";
+      print_atom_list html ch l;
+      output_string ch "}}\n";
+      if html then output_string ch "</pre>\n";
+      output_string ch "\n"
   | Abbrev(s,l) ->
       if needs_output s keys then 
 	begin
 	  if html then begin Html.open_anchor ch s; Html.close_anchor ch end;
+	  if html then output_string ch "<pre>\n";
 	  output_string ch ("@STRING{" ^ s ^ " = ");
-	  print_atom_list html ch keys l;
-	  output_string ch "}\n\n"
+	  print_atom_list html ch l;
+	  output_string ch "}\n";
+	  if html then output_string ch "</pre>\n";
+	  output_string ch "\n"
 	end
   | Entry (entry_type,key,fields) ->
       if needs_output key keys then 
 	begin
 	  if html then begin Html.open_anchor ch key; Html.close_anchor ch end;
+	  if html then output_string ch "<pre>\n";
 	  output_string ch ("@" ^ entry_type ^ "{" ^ key);
 	  List.iter
 	    (fun (field,l) ->
 	       output_string ch (",\n  " ^ field ^ " = ");
-	       if html & field = "CROSSREF" then print_crossref html ch keys l
-	       else print_atom_list html ch keys l)
+	       if html & field = "CROSSREF" then 
+		 print_crossref html ch l
+	       else if html & is_link_field field then
+		 print_link_field ch l
+	       else 
+		 print_atom_list html ch l)
 	    fields;
-	  output_string ch "\n}\n\n"
+	  output_string ch "\n}\n";
+	  if html then output_string ch "</pre>\n";
+	  output_string ch "\n"
+
 	end
 
 let output_bib html ch bib keys =
